#ifndef __ELIB_H__
#define __ELIB_H__

#include <glib.h>
#include <libeina/stream.h>
#include <libeina/tag.h>

#define ELIB_BACKEND "Gstreamer"

#include <gst/gstversion.h>
#define ELIB_BE_VERSION_MAJOR GST_VERSION_MAJOR
#define ELIB_BE_VERSION_MINOR GST_VERSION_MINOR
#define ELIB_BE_VERSION_MICRO GST_VERSION_MICRO

#define ELIB_BE_CHECK_VERSION(major,minor,micro)    \
	(ELIB_BE_VERSION_MAJOR > (major) || \
	(ELIB_BE_VERSION_MAJOR == (major) && ELIB_BE_VERSION_MINOR > (minor)) || \
	(ELIB_BE_VERSION_MAJOR == (major) && ELIB_BE_VERSION_MINOR == (minor) && \
	ELIB_BE_VERSION_MICRO >= (micro)))

typedef struct _EinaPlayer   EinaPlayer;

typedef enum {
	EINA_STATE_PLAY,
	EINA_STATE_PAUSE,
	EINA_STATE_STOP,
} EinaState;

typedef enum {
	EINA_PLAYER_ERROR_SET_INPUT,
	EINA_PLAYER_ERROR_SET_OUTPUT,

	EINA_PLAYER_ERRORS
} EinaPlayerError;

/*
 * New signal handling
 */
typedef enum {
	EINA_SIGNAL_LOAD,
	EINA_SIGNAL_UNLOAD,
	EINA_SIGNAL_START,
	EINA_SIGNAL_END,
	
	EINA_SIGNAL_CHANGE,
	EINA_SIGNAL_PLAY,
	EINA_SIGNAL_PAUSE,
	EINA_SIGNAL_STOP,
	EINA_SIGNAL_SEEK,
	EINA_SIGNAL_VOLUME,
	
	EINA_SIGNAL_ADD,
	EINA_SIGNAL_DEL,
	EINA_SIGNAL_SWAP,
	EINA_SIGNAL_CLEAR,
	EINA_SIGNAL_RANDOM,
	EINA_SIGNAL_REPEAT,

	EINA_SIGNALS
} EinaSignal;

typedef struct EinaSlot {
	gpointer signal[EINA_SIGNALS];
	gpointer user_data;
} EinaSlot;

typedef struct EinaPlug {
	/* General events */
	gpointer load;
	gpointer unload;
	gpointer start;
	gpointer exit;

	/* Core events */
	gpointer change;
	gpointer play;
	gpointer pause;
	gpointer stop;
	gpointer seek;
	gpointer vol;

	/* Playlist events */
	gpointer add;
	gpointer del;
	gpointer swap;
	gpointer clear;
	gpointer random;
	gpointer repeat;

	gpointer volume;

	gpointer user_data;
} EinaPlug;

/* Inicializa el sistema de sonido de eina */
void eina_player_init(int *argc, char **argv[]);

/* Crea un nuevo reproductor con o sin configuracion */
EinaPlayer *eina_player_new(GError **err);
/* Librera todos los recursos asociados al reproductor */
void eina_player_unref(EinaPlayer *p);

gboolean     eina_player_set_input(EinaPlayer *p, gchar *input, GError **err);
const gchar *eina_player_get_input(EinaPlayer *p);
gboolean     eina_player_set_output(EinaPlayer *p, gchar *output, GError **err);
const gchar *eina_player_get_output(EinaPlayer *p);

/* Aade/elimina elementos de la lista de reproduccion */
gboolean eina_player_add      (EinaPlayer *p, EinaStream *stream);
gboolean eina_player_add_uri  (EinaPlayer *p, gchar *uri);
void eina_player_del          (EinaPlayer *p, gint pos);
void eina_player_swap(EinaPlayer *p, gint a, gint b);

/* Vacia la lista de reproduccion y para el reproductor */
void eina_player_clear(EinaPlayer *p);

guint eina_player_slot_add(EinaPlayer *p, EinaSlot slot);
void eina_player_slot_remove(EinaPlayer *p, gint slot_id);

/* Funciones para hacer cambiar al reproductor de estado */
gboolean eina_player_play (EinaPlayer *p);
gboolean eina_player_pause(EinaPlayer *p);
gboolean eina_player_stop (EinaPlayer *p);

/* Funciones para hacer cambiar al reproductor de stream */
gboolean eina_player_next (EinaPlayer *p);
gboolean eina_player_prev (EinaPlayer *p);
gboolean eina_player_nth(EinaPlayer *p, gint pos);

const EinaStream *eina_player_nth_stream(EinaPlayer *p, gint pos);
const EinaStream *eina_player_get_stream(EinaPlayer *p);

gint eina_player_current(EinaPlayer *p);
gint eina_player_total  (EinaPlayer *p);

void eina_player_set_random(EinaPlayer *p, gboolean value);
void eina_player_set_repeat(EinaPlayer *p, gboolean value);

gboolean eina_player_get_random(EinaPlayer *p);
gboolean eina_player_get_repeat(EinaPlayer *p);

gint64 eina_player_get_position  (EinaPlayer *p);
void   eina_player_set_position  (EinaPlayer *p, gint64 pos);
gint64 eina_player_length (EinaPlayer *p);

gboolean eina_player_set_volume(EinaPlayer *p, gint vol);
gint eina_player_get_volume(EinaPlayer *p);

gboolean eina_player_set_mute(EinaPlayer *p, gboolean mute);
gboolean eina_player_get_mute(EinaPlayer *p);

GList * eina_player_get_playlist(EinaPlayer *p);

EinaState eina_player_get_state(EinaPlayer *p);
gboolean eina_player_set_state(EinaPlayer *p);

void eina_player_print_playlist(EinaPlayer *p);
void eina_player_print_random_playlist(EinaPlayer *p);

/*
void eina_player_lock_state(EinaPlayer *p);
void eina_player_unlock_state(EinaPlayer *p);
*/
#endif	
