#ifndef __EINA_STREAM_H__
#define __EINA_STREAM_H__
#include <glib.h>
#include <glib-object.h>
#include <libeina/tag.h>

/* Location or uri, almost always are the same */
#define EINA_STREAM_NONE     0 << 1
#define EINA_STREAM_LOCATION EINA_STREAM_NONE
#define EINA_STREAM_URI      1 << 1

/* Dirname, basename or all(nothing to do) */
#define EINA_STREAM_FULLNAME  1 << 2
#define EINA_STREAM_FILENAME EINA_STREAM_FULLNAME /* Backward compat */
#define EINA_STREAM_DIRNAME   1 << 3
#define EINA_STREAM_BASENAME  1 << 4

/* Other options */
#define EINA_STREAM_UTF8       1 << 5
#define EINA_STREAM_ESCAPE     1 << 6
#define EINA_STREAM_LOCALE     1 << 7
#define EINA_STREAM_URL_DECODE 1 << 8

typedef struct _EinaStream EinaStream;

EinaStream *eina_stream_new(const gchar *uri);
EinaStream *eina_stream_new_from_string(const gchar *str);
void eina_stream_unref(EinaStream *self);
EinaStream *eina_stream_clone(EinaStream *self);

const gchar *eina_stream_get_uri(const EinaStream *self);
const gchar *eina_stream_get_location(const EinaStream *self);
gboolean eina_stream_is_local(const EinaStream *self);
void eina_stream_print(const EinaStream *self);
gboolean eina_stream_format(const EinaStream *self, const gchar *fmt, gint max_fails, gint flags, gchar **dest);

gchar *eina_stream_serialize(EinaStream *self);
EinaStream *eina_stream_unserialize(const gchar *str);

void eina_stream_set_metadata(EinaStream *self, EinaTag tag, gchar *val);
const gchar *eina_stream_get_metadata(const EinaStream *self, EinaTag tag);

#endif

