#include <libeina/config.h>
#include <string.h>

struct _EinaConfig {
	GHashTable *values;
	GSList *callbacks, *datas;
};

/*
 * Private
 */
gboolean eina_config_hash_equal_func (gconstpointer a, gconstpointer b);

/*
 * Implementation
 */
EinaConfig *eina_config_new (void) {
	EinaConfig *tmp;
	tmp = g_new0(EinaConfig, 1);
	tmp->values = g_hash_table_new_full(g_str_hash, eina_config_hash_equal_func, NULL, g_free);
	tmp->callbacks = NULL;
	tmp->datas = NULL;
	return tmp;
}

void eina_config_free (EinaConfig *cfg) {
	g_hash_table_destroy(cfg->values);
	g_slist_free(cfg->callbacks);
	g_slist_free(cfg->datas);
	g_free(cfg);
}

gboolean eina_config_hash_equal_func (gconstpointer a, gconstpointer b) {
	if ( g_ascii_strcasecmp( (gchar *) a, (gchar *) b) )
		return FALSE;
	else
		return TRUE;
}

void eina_config_set(EinaConfig *cfg, gchar *key, GValue *val) {
	void (*callback)();
	gchar *_key;
	// GValue *old_value;
	
	gpointer data;
	guint i;
	
	_key = g_strdup(key);
	/* If key is unset, this can segfault
	old_value = g_new0(GValue, 1);
	old_value = g_value_init(old_value, eina_config_get_type(cfg, key));
	g_value_copy(old_value, eina_config_get(cfg, key));
	*/
	
	g_hash_table_insert(cfg->values, (gpointer) _key, (gpointer) val);
	
	for ( i = 0; i < g_slist_length(cfg->callbacks); i++) {
		callback = g_slist_nth_data(cfg->callbacks, i);
		data = g_slist_nth_data(cfg->datas, i);

		/* callback(cfg, key, old_value, val, data); */
		callback(cfg, key, NULL, val, data);
	}
	/* g_free(old_value); */
}

void eina_config_set_bool (EinaConfig *cfg, gchar *key, gboolean val) {
	GValue   *_val;

	_val = g_new0(GValue, 1);
	_val = g_value_init(_val, G_TYPE_BOOLEAN);
	g_value_set_boolean(_val, val);
					
	eina_config_set(cfg, key, _val);
}

void eina_config_set_int (EinaConfig *cfg, gchar *key, gint val) {
	GValue   *_val;

	_val = g_new0(GValue, 1);
	_val = g_value_init(_val, G_TYPE_INT);
	g_value_set_int(_val, val);

	eina_config_set(cfg, key, _val);
}

void eina_config_set_float (EinaConfig *cfg, gchar *key, gfloat val) {
	GValue   *_val;

	_val = g_new0(GValue, 1);
	_val = g_value_init(_val, G_TYPE_FLOAT);
	g_value_set_float(_val, val);
					
	eina_config_set(cfg, key, _val);
}

void eina_config_set_string (EinaConfig *cfg, gchar *key, gchar *val) {
	GValue   *_val;

	_val = g_new0(GValue, 1);
	_val = g_value_init(_val, G_TYPE_STRING);
	g_value_set_string(_val, g_strdup(val));

	eina_config_set(cfg, key, _val);
}

/*
 * Getters
 */ 
GValue *eina_config_get (EinaConfig *cfg, gchar *key) {
	return g_hash_table_lookup(cfg->values, key);
}

gint eina_config_get_int (EinaConfig *cfg, gchar *key, gint def) {
	GValue *value = NULL;
	
	if ( cfg->values != NULL )
		value = eina_config_get(cfg, key);
	
	if ( value == NULL ) {
		eina_config_set_int(cfg, key, def);
		return def;
	}
	return g_value_get_int(value);
}

gfloat eina_config_get_float (EinaConfig *cfg, gchar *key, gfloat def) {
	GValue *value = NULL;
	
	if ( cfg->values != NULL )
		value = eina_config_get(cfg, key);

	if ( value == NULL ) {
		eina_config_set_float(cfg, key, def);
		return def;
	}
	return g_value_get_float(value);
}

gboolean eina_config_get_bool (EinaConfig *cfg, gchar *key, gboolean def) {
	GValue *value = NULL;
	
	if ( cfg->values != NULL )
		value = eina_config_get(cfg, key);
	
	if ( value == NULL ) {
		eina_config_set_bool(cfg, key, def);
		return def;
	}
	return g_value_get_boolean(value);
}

const gchar * eina_config_get_string (EinaConfig *cfg, gchar *key, gchar *def) {
	GValue *value = NULL;
	
	if ( cfg->values != NULL )
		value = eina_config_get(cfg, key);
	
	if ( value == NULL ) {
		eina_config_set_string(cfg, key, def);
		return def;
	}
	return g_value_get_string(value);
}

/*
 * Get info about types
 */
GType eina_config_get_type(EinaConfig *cfg, gchar *key) {
	return G_VALUE_TYPE(eina_config_get(cfg, key));
}

const gchar *eina_config_get_type_name(EinaConfig *cfg, gchar *key) {
	return G_VALUE_TYPE_NAME(eina_config_get(cfg, key));
}

/*
 * Notifier
 */
gint eina_config_notify_add(EinaConfig *cfg, gpointer callback, gpointer data) {
	GSList *tmp;
	
	cfg->callbacks = g_slist_append(cfg->callbacks, callback);
	cfg->datas     = g_slist_append(cfg->datas, data);

	tmp = g_slist_find(cfg->callbacks, callback);
	return g_slist_position(cfg->callbacks, tmp);
}

void eina_config_notify_remove(EinaConfig *cfg, gint id) {
	GSList *node;

	node = g_slist_nth(cfg->callbacks, id);
	cfg->callbacks = g_slist_remove(cfg->callbacks, node);

	node = g_slist_nth(cfg->datas, id);
	cfg->datas = g_slist_remove(cfg->datas, node);
}

/*
 * Misc functions
 */
GSList *eina_config_keys (EinaConfig *cfg) {
	GSList *keys = NULL;
	void eina_config_push_key (gpointer key, gpointer value, gpointer data) {
		keys = g_slist_append(keys, key);
	}
	g_hash_table_foreach(cfg->values, eina_config_push_key, NULL);
	return keys;
}

