#include <libeina/player.h>
#include <libeina/player-priv.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <strings.h>
#include <glib/gprintf.h>

/* Internal function, sets EOS so that the watcher can see it XXX Ugly hack? */
void eina_player_trap_eos(gpointer data1, gpointer data2) {
	((EinaPlayer *) data2)->eos = TRUE;
}

/*
 * This function is called periodically
 * to check for EOS and act accordingly
 */
gboolean eina_player_eos_watcher (gpointer data) {
	EinaPlayer *p = data;
#if 0
	gint64 current_pos, current_len;
	gint64 diff;

	current_pos = eina_player_get_position(p);
	current_len = eina_player_length(p);
	diff        = current_len - current_pos;

	/* Chech if we have to launch the fadeout */
	if ((current_len != 0) && (current_pos != 0) && FALSE) {
		if ((diff < 5000000000LL) && p->fadeout_id == 0) {
			g_printf("Starting fadeout\n");
			p->fadeout_id = eina_player_start_fadeout(p, 5000000000LL);
		} else if ( p->fadeout_id ) {
			/* Cancel fadeout */
			g_printf("Canceling fadeout\n");
			eina_player_stop_fadeout(p);
		}
	}
#endif

	if ( p->eos ) {
		p->eos = FALSE;

		/* Stop player if we cannot advance */
		if ( ! eina_player_next(p) ) {
			eina_player_stop(p);
		} else {
			eina_player_play(p);
		}
	}

	/* Return TRUE so that we can be called again */
	return TRUE;
}

guint eina_player_start_fadeout(EinaPlayer *p, gint64 length) {
	p->fadeout                = g_new0(EinaPlayerFade, 1);
	p->fadeout->orig_volume   = eina_player_get_volume(p);
	p->fadeout->orig_position = eina_player_get_position(p);
	p->fadeout->length        = length;

	return g_timeout_add(100, eina_player_fade_out, p);
}

void eina_player_stop_fadeout(EinaPlayer *p) {
	g_source_remove(p->fadeout_id);
	g_free(p->fadeout);
}

gboolean eina_player_fade_out(gpointer data) {
	gint64 current_position, diff;
	gint   percent = 0;
	EinaPlayer     *p    = (EinaPlayer *) data;
	EinaPlayerFade *fade = p->fadeout;

	current_position = eina_player_get_position(p);
	diff = fade->orig_position - current_position;

	g_printf("orig: %lld current: %lld diff: %lld, percent: %f\n", 
					(long long int)fade->orig_position, 
					(long long int)current_position, 
					(long long int)diff, 
					0.0);

	if ( percent == 0 ) {
		g_free(fade);
		return FALSE;
	}
	else
		return TRUE;
}

void _eina_player_emit_signal(EinaPlayer *p, EinaSignal signal, gpointer data1, gpointer data2) {
	gint i;
	void (*func)();
	EinaSlot *slot;
	gpointer user_data;

	for ( i = 0; (slot = (EinaSlot *) g_list_nth_data(p->slots, i)); i++) {
		func = slot->signal[signal];
		user_data = slot->user_data;

		if ( func == NULL )
			continue;

		switch ( signal ) {
			case EINA_SIGNAL_ADD:
				/* f(EinaStream *new_stream, gint pos, gpointer data) */
				func(p, data1, data2, user_data);
				break;

			case EINA_SIGNAL_DEL:
				/* f(EinaStream *deleted_stream, gint pos, gpointer data) */
				func(p, data1, data2, user_data);
				break;

			case EINA_SIGNAL_PLAY:
				/* f(EinaStream *current_stream, gint current, gpointer data); */
				func(p, data1, data2, user_data);
				break;
			
			case EINA_SIGNAL_PAUSE:
				/* f(gboolean mute, gpointer data) */
				func(p, data1, user_data);
				break;
			
			case EINA_SIGNAL_STOP:
				/* f(gpointer data) */
				func(p, user_data);
				break;

			case EINA_SIGNAL_CHANGE:
				/* f(gint old_pos, gint new_pos, gpointer data) */
				func(p, data1, data2, user_data);
				break;
				
			case EINA_SIGNAL_SWAP:
				/* f(gint pos_stream_a, gint pos_stream_b, data */
				func(p, data1, data2, user_data);
				break;
				
			case EINA_SIGNAL_CLEAR:
				/* f(gpointer data) */
				func(p, user_data);
				break;
				
			case EINA_SIGNAL_VOLUME:
				/* f(gint old_vol, gint new_vol, gpointer data) */
				func(p, data1, data2, user_data);
				// func(p, *(gfloat *) data1, *(gfloat *) data2, user_data)
				break;
				
			case EINA_SIGNAL_SEEK:
				/* f(gint64 new_pos, gpointer data) */
				func(p, *(gint64 *) data1, *(gint64 *) data2, user_data);
				break;
				
			case EINA_SIGNAL_REPEAT:
				/* f(gboolean repeat, data) */
				func(p, data1, user_data);
				break;

			case EINA_SIGNAL_RANDOM:
				/* f(gboolean random, data) */
				func(p, data1, user_data);
				break;
				
			default:
				break;
		}
	}
}

const gchar *eina_priv_get_mime(EinaPlayer *p, const gchar *filename) {
	const gchar *mimetype;
	gchar *pos;
	gchar tokens[] = ",;:";
	gint i;

	mimetype = magic_file(p->mimedetector, filename);
	if ( mimetype == NULL ) return NULL; /* File is unreadable or unrecogniced type*/

	for ( i = 0; i < strlen(tokens); i++ ) {
		pos = index(mimetype, tokens[i]);
		if ( pos != NULL ) pos[0] = '\0';
	}

	return mimetype;
}

#if 0
gboolean eina_player_fadder_watcher(gpointer data) {
	EinaPlayer *p = data;
	gint64 current_pos, current_len;
	gint64 diff;
	static gint volume = -1;
	gint newvolume;
	gfloat percent;
	
	current_pos = eina_player_get_position(p);
	current_len = eina_player_length(p);
	diff = current_len - current_pos;
	
	if ( current_len == 0 || current_pos == 0 )
		return TRUE;

	if ( diff < 5000000000LL ) {
		/*
		 * - We are called 30 times until eos
		 * - Volume range is [0,100]
		 */
		if ( volume == -1 )
			volume = eina_player_get_volume(p);

		percent = (diff / 10000000LL) / (gfloat) 500;
		g_printf("diff is: %lld percent: %f\n",  diff, percent);
		newvolume = volume * percent;
		g_printf("new vol %d\n", newvolume);
		eina_player_set_volume(p, newvolume);
	} else
		volume = -1;

	return TRUE;
}

/* Save player state */
inline void eina_player_lock_state (EinaPlayer *p) {
	p->state = eina_core_get_state(p->core);
}

/* Restore saved player state */
inline void eina_player_unlock_state (EinaPlayer *p) {
	eina_core_set_state(p->core, p->state);
}
#endif
